/*******************************************************************************
 *
 *		Azoteq IQS263_Example_Code
 *
 *              IQS263_Handler - IQS263 device settings and events
 *
*******************************************************************************/

#include "Includes.h"

/***************************** GLOBAL VARIABLES *******************************/

unsigned char touch0 = 0;                       // variable identified with a Touch event
unsigned char prox = 0;                         // variable identified with a Prox event
unsigned char sliderCoords;                     // variable identified with a Slide event
unsigned char movement;                         // variable identified with a Movement event
unsigned char tap = 0;                          // variable identified with a Tap event
unsigned char leftFlick;                        // variable identified with a Flick event(left)
unsigned char rightFlick;                       // variable identified with a Flick event(right)
unsigned char events = 0;                       // variable identified with all events

// Data buffer for communication with the IQS263, set to an array size of 30 elememnts,
// which can fit all the IQS263 settings.
unsigned char data_buffer[30];

/*************************** IQS263 DEVICE SETTINGS ***************************/

void IQS263DeviceSettings(void)
{
    // Switch on all lights to show whether the Program
    // has entered the DeviceSettings function.
    all_lights_on();

    // Read the product number
    CommsIQS_start();
    CommsIQS_Read(IQS263_ADDR, DEVICE_INFO, &data_buffer[0], 2);
    CommsIQS_stop();

    // Switch the IQS263 into projection mode - if necessary
    data_buffer[0] = SYSTEM_FLAGS_VAL;
    CommsIQS_start();
    CommsIQS_Write(IQS263_ADDR, SYS_FLAGS, &data_buffer[0], 1);
    CommsIQS_stop();

    // Set active channels
    data_buffer[0] = ACTIVE_CHS;
    CommsIQS_start();
    CommsIQS_Write(IQS263_ADDR, ACTIVE_CHANNELS, &data_buffer[0], 1);
    CommsIQS_stop();

    // Setup touch and prox thresholds for each channel
    data_buffer[0] = PROX_THRESHOLD;
    data_buffer[1] = TOUCH_THRESHOLD_CH1;
    data_buffer[2] = TOUCH_THRESHOLD_CH2;
    data_buffer[3] = TOUCH_THRESHOLD_CH3;
    data_buffer[4] = MOVEMENT_THRESHOLD;
    data_buffer[5] = RESEED_BLOCK;
    data_buffer[6] = HALT_TIME;
    data_buffer[7] = I2C_TIMEOUT;
    CommsIQS_start();
    CommsIQS_Write(IQS263_ADDR, THRESHOLDS, &data_buffer[0], 8);
    CommsIQS_stop();

    // Set the ATI Targets (Target Counts)
    data_buffer[0] = ATI_TARGET_TOUCH;
    data_buffer[1] = ATI_TARGET_PROX;
    CommsIQS_start();
    CommsIQS_Write(IQS263_ADDR, TIMINGS_AND_TARGETS, &data_buffer[0], 2);
    CommsIQS_stop();

    // Set the BASE value for each channel
    data_buffer[0] = MULTIPLIERS_CH0;
    data_buffer[1] = MULTIPLIERS_CH1;
    data_buffer[2] = MULTIPLIERS_CH2;
    data_buffer[3] = MULTIPLIERS_CH3;
    CommsIQS_start();
    CommsIQS_Write(IQS263_ADDR, MULTIPLIERS, &data_buffer[0], 4);
    CommsIQS_stop();

    // Setup prox settings
    data_buffer[0] = PROXSETTINGS0_VAL;
    data_buffer[1] = PROXSETTINGS1_VAL;
    data_buffer[2] = PROXSETTINGS2_VAL;
    data_buffer[3] = PROXSETTINGS3_VAL;
    data_buffer[4] = EVENT_MASK_VAL;
    CommsIQS_start();
    CommsIQS_Write(IQS263_ADDR, PROX_SETTINGS, &data_buffer[0], 5);
    CommsIQS_stop();

    // Setup Compensation (PCC)
    data_buffer[0] = COMPENSATION_CH0;
    data_buffer[1] = COMPENSATION_CH1;
    data_buffer[2] = COMPENSATION_CH2;
    data_buffer[3] = COMPENSATION_CH3;
    CommsIQS_start();
    CommsIQS_Write(IQS263_ADDR, COMPENSATION, &data_buffer[0], 4);
    CommsIQS_stop();

    // Set timings on the IQS263
    data_buffer[0] = LOW_POWER;
    CommsIQS_start();
    CommsIQS_Write(IQS263_ADDR, TIMINGS_AND_TARGETS, &data_buffer[0], 1);
    CommsIQS_stop();

    // Set gesture timers on IQS263
    data_buffer[0] = TAP_TIMER;
    data_buffer[1] = FLICK_TIMER;
    data_buffer[2] = FLICK_THRESHOLD;
    CommsIQS_start();
    CommsIQS_Write(IQS263_ADDR, GESTURE_TIMERS, &data_buffer[0], 3);
    CommsIQS_stop();

    // Redo ati
    data_buffer[0] = 0x10;
    CommsIQS_start();
    CommsIQS_Write(IQS263_ADDR, PROX_SETTINGS, &data_buffer[0], 1);
    CommsIQS_stop();

    // Wait untill the ATI algorithm is done
    do
    {
        delay_ms(10);
        CommsIQS_start();
        CommsIQS_Read(IQS263_ADDR, SYS_FLAGS, &data_buffer[0], 1);
        CommsIQS_stop();
    }
    while ((data_buffer[0] & 0b00000100) == 0b00000100);

    // read the error bit to determine if ATI error occured
    CommsIQS_start();
    CommsIQS_Read(IQS263_ADDR, PROX_SETTINGS, &data_buffer[0], 2);
    CommsIQS_stop();

    // If an ATI occured flash the LEDS
    if (data_buffer[1] & 0x02 == 0x02)
    {
        int i = 0;
        for (i = 0; i < 10; i++) {
            all_lights_on();
            delay_ms(50);
            all_lights_off();
        }
    }

    // Switch all the LEDS off to indicate that ATI was successful
    all_lights_off();
}
/*******************************************************************************
 *             This function will display the different event
 *             features of the IQS263 IC. This will be done by
 *             toggeling of four different LEDS, to create
 *             patterns identified with each event.
 ******************************************************************************/

void handleEvents(void)
{
    CommsIQS_start();                                                           // Start the communication session
    CommsIQS_Read(IQS263_ADDR, SYS_FLAGS, &data_buffer[0], 2);                  // Read the system flags register to enable all events
    CommsIQS_repeat_start();                                                    // Create another start condition
    CommsIQS_Read(IQS263_ADDR, TOUCH_BYTES, &data_buffer[2], 1);                // Read from the touch bytes register to enable touch events
    CommsIQS_repeat_start();                                                    // Create another start condition
    CommsIQS_Read(IQS263_ADDR, COORDINATES, &data_buffer[3], 3);                // Read the coordinates register to get slider coordinates
    CommsIQS_stop();                                                            // Stop the communication session

    events = data_buffer[1];

    /******************************* PROXIMITY ********************************/

    if ((events & 0x01) == 0x01)
        proxEvent();

    /******************************** TOUCH ***********************************/
    if ((events & 0x02) == 0x02)
        touchEvent();

    /******************************** SLIDE ***********************************/

    if (((events & 0x04) == 0x04) && (events != 0))
        slideEvent();

    /******************************* MOVEMENT *********************************/

    if ((events & 0x10) == 0x10)
        movementEvent();
    else
        LATDbits.LATD2 = 1;

    /********************************* TAP ************************************/

    if((events & 0x20) == 0x20)
        tapEvent();

    /******************************* FLICK (LEFT) *****************************/

    if((events & 0x40) == 0x40)
        flickLeft();

    /******************************* FLICK (RIGHT) ****************************/

    if((events & 0x80) == 0x80)
        flickRight();

}

/*******************************************************************************
 *      This function handles touch events. If a touch event occurs
 *      an LED will be switched on marking the current channel that the
 *      touch event occured in.
 ******************************************************************************/
void touchEvent(void)
{
    touch0 = data_buffer[2];

    if (touch0 != 0)
    {
        /* CHANNEL 1*/
        if ((touch0 & 0x02) == 0x02)                    // If a touch event occurs on Channel 1
        {
            LATDbits.LATD0 = 0;                         // Toggle LED 1 ON
        }
        else
        {
            LATDbits.LATD0 = 1;                         // Toggle LED 1 OFF
        }

        /* CHANNEL 2 */
        if ((touch0 & 0x04) == 0x04)                    // If a touch event occurs on Channel 2
        {
            LATDbits.LATD1 = 0;                         // Toggle LED 2 ON
        }
        else
        {
            LATDbits.LATD1 = 1;                         // Toggle LED 2 OFF
        }

        /* CHANNEL 3 */
        if ((touch0 & 0x08) == 0x08)                    // If a touch event occurs on Channel 3
        {
            LATDbits.LATD0 = 0;                         // Toggle LED 1 ON
            LATDbits.LATD1 = 0;                         // Toggle LED 2 ON
        }
        else
        {
            LATDbits.LATD0 = 1;                         // Toggle LED 1 OFF
            LATDbits.LATD1 = 1;                         // Toggle LED 2 OFF
        }
    }
}
/*******************************************************************************
 *      This function handles a slide event, if a slide event is triggered
 *      an LED will switch on marking the position that the finger of the
 *      user is currently on including the coordinate value thereof.
 ******************************************************************************/
void slideEvent(void)
{
    sliderCoords = data_buffer[3];
    touch0 = data_buffer[2];

    if (touch0 != 0)
    {
        if ((sliderCoords > 0 && sliderCoords < 85)&& ((touch0 & 0x02) == 0x02))
        {
            LATDbits.LATD0 = 0;             // Toggle LED 1 ON
            LATDbits.LATD1 = 1;             // Toggle LED 2 OFF
            LATDbits.LATD2 = 1;             // Toggle LED 3 OFF
            LATDbits.LATD3 = 1;             // Toggle LED 4 OFF
        }
        else if ((sliderCoords > 0 && sliderCoords < 170)&& ((touch0 & 0x04) == 0x04))
        {
            LATDbits.LATD0 = 1;             // Toggle LED 1 OFF
            LATDbits.LATD1 = 0;             // Toggle LED 2 ON
            LATDbits.LATD2 = 1;             // Toggle LED 3 OFF
            LATDbits.LATD3 = 1;             // Toggle LED 4 OFF
        }
        else if ((sliderCoords > 170 && sliderCoords < 255)&& ((touch0 & 0x08) == 0x08))
        {
            LATDbits.LATD0 = 0;             // Toggle LED 1 ON
            LATDbits.LATD1 = 0;             // Toggle LED 2 ON
            LATDbits.LATD2 = 1;             // Toggle LED 3 OFF
            LATDbits.LATD3 = 1;             // Toggle LED 4 OFF
        }
        else
        {
            all_lights_off();               // Toggle All LEDS OFF
        }
    }
}

/*******************************************************************************
 *      This function handles a prox event, if proximity is being detected
 *      LED 4 will be switched ON, and switched OFF if no proximity is being
 *      detected.
 ******************************************************************************/
void proxEvent(void)
{
    prox = data_buffer[2];

    if ((prox & 0x01) == 0x01)              // If a prox event occures
    {
        LATDbits.LATD3 = 0;                 // Toggle LED 4 ON
    }
    else
    {
        LATDbits.LATD3 = 1;                 // Toggle LED 4 OFF
    }
}

/*******************************************************************************
 *      This function handles a movement event, each time movement is
 *      sensed, LED 3 will be switched ON, and switched OFF if no movement
 *      is sensed.
 ******************************************************************************/
void movementEvent(void)
{
    movement = data_buffer[1];

    if ((movement & 0x10) == 0x10)          // If a movement event occurs
    {
        LATDbits.LATD2 = 0;                 // Toggle LED 3 ON
    }
    else
    {
        LATDbits.LATD2 = 1;                 // Toggle LED 3 OFF
    }
}

/*******************************************************************************
 *      This function handles a tap event, each time this event is
 *      triggered ALL LEDS will flash 3 times.
 ******************************************************************************/
void tapEvent(void)
{
    UINT8 i;
    tap = data_buffer[1];

    if ((tap & 0x20) == 0x20)               // If a tap event occurs
    {
        for (i = 0; i < 3; i++)
        {
            LATDbits.LATD0 = 0;
            LATDbits.LATD1 = 0;            /* Toggle ALL LEDS ON */
            LATDbits.LATD2 = 0;
            LATDbits.LATD3 = 0;
            delay_ms(50);                  /* Wait for 50ms */
            LATDbits.LATD0 = 1;
            LATDbits.LATD1 = 1;
            LATDbits.LATD2 = 1;            /* Toggle ALL LEDS OFF */
            LATDbits.LATD3 = 1;
            delay_ms(50);                  /* Wait for 50ms */
        }
    }
}

/*******************************************************************************
 *     This function handles a right flick event, each time this event is
 *     is triggered, LED 1 will flicker 5 times.
 ******************************************************************************/
void flickRight(void)
{
    UINT8 i;
    rightFlick = data_buffer[1];

    if ((rightFlick & 0x80) == 0x80)       // If a right flick event occurs
    {
        for (i = 0; i < 5; i++)
        {
            tap = 0x00;                     // avoid tap event coliding with flicker event
            LATDbits.LATD0 = 0;             // Toggle LED 1 ON
            delay_ms(300);                  // Wait 300ms
            LATDbits.LATD0 = 1;             // Toggle LED 1 OFF
            delay_ms(300);                  // Wait 300ms
        }
    }
    else
    {
        LATDbits.LATD0 = 1;                 // Toggle LED 1 OFF
    }
}

/*******************************************************************************
 *     This function handles a left flick event, each time this event is
 *     is triggered, LED 2 will flicker 5 times.
 ******************************************************************************/
void flickLeft(void)
{
    UINT8 i;
    leftFlick = data_buffer[1];

    if ((leftFlick & 0x40) == 0x40)        // If a left click event occurs
    {
        for (i = 0; i < 5; i++)
        {
            tap = 0x00;                     // avoid tap event coliding with flicker event
            LATDbits.LATD1 = 0;             // Toggle LED 2 ON
            delay_ms(300);                  // Wait 300ms
            LATDbits.LATD1 = 1;             // Toggle LED 2 OFF
            delay_ms(300);                  // Wait 300ms
        }
    }
    else
    {
        LATDbits.LATD1 = 1;                 // Toggle LED 2 OFF
    }
}
















