/*********************************************************************************************************************

		Azoteq IQS263_Example_Code

                i2c_low_level.c - i2c low level functions

********************************************************************************************************************/

#include "Includes.h"

/*  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	Description:	Initialises the SPI module on the PIC18F4550

*/
void Comms_init(void)		     //Initiates i2c module on PIC18F4550
{
    SSPADD = 0x1C;		     // settings for I2C frequency - 416kHz
    SSPSTAT |= 0x80;             // slew rate control for high speed (400kHz)

    TRISB = TRISB | 0x03;        //set TRISB<0> Make I2c SDA and SCL inputs

    PIR1bits.SSPIF = 0;          // reset the initial interrupt vector
    SSPCON1 = 0x28;              //enables i2c, set to master

    LATA = LATA | 0x08;          //switch on the IQS _ should comment this out again
}

/*  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
						----    CommsIQS_send()    ----
	Description:	Transmits the data byte given as parameter via the I2C of the PIC18F4550
				Note that the I2C communication channel must already be active before calling this function,
				as no start bits are included in this function

	Parameters:		send_data - data byte to be transmitted via the I2C

*/
void CommsIQS_send(unsigned char send_data)
{
    SSPBUF = send_data;                     // write transmit byte to buffer
    while (PIR1bits.SSPIF == 0)             // wait for transmit complete flag
    {}//empty
    PIR1bits.SSPIF = 0;                     //clear flag

    while (SSPCON2bits.ACKSTAT == 1)        // acknowledge was not yet received from slave
    {}//empty}
}

/*  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
						----    CommsIQS_read_ack()    ----
	Description:	Enables the Master Receive Mode of the I2C module on the PIC18F4550. The data received is
				returned and an (ACK) acknowledge is sent to the IQS to indicate that more byte read's will
				still take place.

	Return:		The data received via the I2C

*/

unsigned char CommsIQS_read_ack(void)
{
    unsigned char temp;

    SSPCON2bits.RCEN = 1;		// enable master receiver mode
    while (PIR1bits.SSPIF == 0)	// wait for byte received flag
    {}
    PIR1bits.SSPIF = 0;		//clear flag

    while (SSPSTATbits.BF == 0)	// wait for buffer full flag (receive complete)
    {}
    temp = SSPBUF;			// store received byte

    SSPCON2bits.ACKDT = 0;		// enable ACK
    SSPCON2bits.ACKEN = 1;		// execute ACK sequence

    while (PIR1bits.SSPIF == 0)	// Wait for ACK transmission complete
    {}
    PIR1bits.SSPIF = 0;		//clear flag

    while (SSPCON2bits.ACKEN == 1) 	//verify acknowledge sequence is complete
    {}

    return temp;
}

/*  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
						----    CommsIQS_read_nack()    ----
	Description:	Enables the Master Receive Mode of the I2C module on the PIC18F4550. The data received is
				returned and a NACK acknowledge is sent to the IQS to indicate that this was the final read
				of the current continuous read block.

	Return:		The data received via the I2C

*/
unsigned char CommsIQS_read_nack(void)
{
    unsigned char temp;

    SSPCON2bits.RCEN = 1;		// enable master receiver mode
    while (PIR1bits.SSPIF == 0)     // wait for byte received flag
    {}
    PIR1bits.SSPIF = 0;		//clear flag

    while (SSPSTATbits.BF == 0)     // wait for buffer full flag (receive complete
    {}
    temp = SSPBUF;                  // store received byte

    SSPCON2bits.ACKDT = 1;          // enable NACK
    SSPCON2bits.ACKEN = 1;          // execute NACK sequence

    while (PIR1bits.SSPIF == 0)     // Wait for NACK transmission complete
    {}
    PIR1bits.SSPIF = 0;		//clear flag

    while (SSPCON2bits.ACKEN == 1) 	//verify acknowledge sequence is complete
    {}

    return temp;
}

/*  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
						----    CommsIQS_start()    ----
				A i2c START condition is generated

*/

void CommsIQS_start(void)
{
    // IQ263 starts in full streaming mode
    while (PORTAbits.RA1 == 1)                      //wait for IQS device to become ready
    {}

    SSPCON2bits.SEN = 1;                            //start condition

    while (PIR1bits.SSPIF == 0)                     //wait for start condition to be generated
    {}
    PIR1bits.SSPIF = 0;                             //clear flag

    while (SSPCON2bits.SEN == 1)                    // verify start is complete
    {}
}

/*  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
						----    CommsIQS_repeat_start()    ----
	Description:	An i2c START condition is generated.

*/

void CommsIQS_repeat_start(void)
{
    SSPCON2bits.RSEN = 1;			//start condition

    while (PIR1bits.SSPIF == 0)             //wait for start to be generated
    {}

    PIR1bits.SSPIF = 0;			//clear flag

    while (SSPCON2bits.RSEN == 1)           //verify that restart is complete
    {}
    //success
}

/*  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
						----    CommsIQS_stop()    ----
	Description:	An i2c STOP condition is generated.

*/

void CommsIQS_stop(void)
{
    SSPCON2bits.PEN = 1;                            //stop condition

    while (PIR1bits.SSPIF == 0)                     //wait for stop condition to be generated
    {}
    PIR1bits.SSPIF = 0;                             //clear flag

    while (SSPCON2bits.PEN == 1)                    // verify stop is complete
    {}

    //wait for the IQS device to become ready
    while(PORTAbits.RA1 == 1)                       //wait for the IQS to change from ready to not ready
    {}
    while(PORTAbits.RA1 == 0)                       // wait for the IQS to change from not ready to ready
    {}
    //wait for IQS device to become ready
}
